@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Secure Locate v1.2                                                        *
+ * Secure Locate v1.3                                                        *
  * October 9, 1998                                                           *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.          *
@@ -49,9 +49,10 @@
 
 /* GLOBALS */
 
-#define VERSION "Secure Locate v1.2 - Released October 14, 1998\n"
+#define VERSION "Secure Locate v1.3 - Released October 14, 1998\n"
 #define SLOCATEDB "/var/lib/slocate/slocate.db"
 #define TMPSLOCATEDB "/var/lib/slocate/slocate.tmp"
+#define SLOCATEDB_DIR "/var/lib/slocate/"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC 0x80
 #define SLOC_GRP "slocate"
@@ -76,7 +77,7 @@ unsigned short SLOC_GID=65534; /* Seems to be the default nobody gid */
 void
 usage() 
 {
-   printf("Secure Locate v1.2\n"
+   printf("Secure Locate v1.3\n"
           "Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.\n"
           "\n"
           "usage: %s [options] <search string>\n\n"
@@ -134,40 +135,37 @@ get_short (FILE *fp)
    return (x << 8) | (fgetc (fp) & 0xff);
 }
 
+/* Make sure that the DB Directory exists */
+
+int
+check_dir()
+{
+	   int res=1;
+	   struct stat tmpstat;
+	
+	   if (lstat(SLOCATEDB_DIR,&tmpstat) == -1) {
+			      fprintf(stderr,"%s: error accessing DB Directory: %s : %s\n",progname,SLOCATEDB_DIR,strerror(errno));
+			      res=0;
+		}
+	
+	   return(res);
+}
+
 /* Get the GID for group slocate */
 
 unsigned short
 get_gid()
 {
    unsigned short GGID=SLOC_GID;
-   int foundit=0;
-	int filefound=0;
 	struct group *grpres;
 	
-   setgrent();
-	while ((grpres = (struct group *)getgrent()) != NULL) {
-		filefound=1;
-		if (!strcmp(grpres->gr_name,SLOC_GRP)) {
-			GGID = grpres->gr_gid;
-			foundit=1;			
-		}
-	}
-	
-	if (filefound)
-		 endgrent();
-	
-	if (!foundit && filefound) {
-      fprintf(stderr,"%s: Could not find the group: %s in the: %s file.\n",progname,SLOC_GRP,GRPFILE);
+	if ((grpres = getgrnam(SLOC_GRP)) == NULL) {		
+      fprintf(stderr,"%s: Could not find the group: %s in the /etc/group file.\n",progname,SLOC_GRP);
       fprintf(stderr,"This is a result of the group missing or a corrupted group file.\n");
       exit(1);
-   }	
-	
-   if (!filefound) {
-      fprintf(stderr,"%s: Could not open %s for reading.\n\n"
-              "Read the README.DOC file in the Secure Locate source directory.\n",progname,GRPFILE);
-      exit(1);
-   }
-	
+	}
+
+	GGID = grpres->gr_gid;
    
    return GGID;
 }
@@ -177,7 +175,7 @@ get_gid()
 char *
 parse_exclude(char *estr)
 {
-   char *excludestr;
+   char *excludestr=NULL;
    char *ptr1;
    char *ptr2=NULL;
    int grow=1;
@@ -200,14 +198,16 @@ parse_exclude(char *estr)
       }
 
       grow += (ptr2-ptr1)+2;
-      excludestr = (char *)realloc(excludestr,grow);
+      excludestr = realloc(excludestr,grow);		
       strcat(excludestr,"*");
       strncat(excludestr,ptr1,ptr2-ptr1);
       strcat(excludestr,"*");
       if (ptr2[0] == ',')
           ptr1 = ptr2+1;
    }
-      
+  
+	printf("E: %s\n",excludestr);
+	exit(1);
    return (excludestr);
 }
 
@@ -216,7 +216,7 @@ match_exclude(char *path,char *filedir) {
    int res=0;
    char *newstr;
    
-   newstr = (char *)statmalloc(strlen(path)+strlen(filedir)+3);
+   newstr = statmalloc(strlen(path)+strlen(filedir)+3);
    newstr[0] = '\0';
    strcat(newstr,"*");
    strcat(newstr,path);
@@ -273,7 +273,7 @@ load_dir(int dir_fd, dir_item *listptr)
          
          /* If directory is emtpy then add a space to listptr */         
          if (filename == NULL) {
-            listptr->name = (char *)statmalloc(1);
+            listptr->name = statmalloc(1);
             listptr->name[0] = '\0';
             listptr->empty = 1;
             listptr->type = 0;
@@ -291,7 +291,7 @@ load_dir(int dir_fd, dir_item *listptr)
       
       
       /* Add information from filenames->d_name to listptr */
-      listptr->name = (char *)statmalloc(strlen(filename->d_name)+1);
+      listptr->name = statmalloc(strlen(filename->d_name)+1);
       listptr->name[0] = '\0';
       strcat(listptr->name,filename->d_name);
       listptr->name_len = strlen(listptr->name);      
@@ -338,25 +338,25 @@ get_cur_dir(dir_item *listptr, char *dirstr)
       items++;
    }
 
-   dirarray = (char **)calloc(items,sizeof(char *));
+   dirarray = calloc(items,sizeof(char *));
    
    dir_ptr=listptr->left;
    
    for (i = items-1; i != -1; i--) {
-      dirarray[i] = (char *)malloc(strlen(dir_ptr->name)+1);
+      dirarray[i] = malloc(strlen(dir_ptr->name)+1);
       dirarray[i][0] = '\0';
       strcat(dirarray[i],dir_ptr->name);
 
       dir_ptr=dir_ptr->left;
    }
 
-   cur_dir = (char *) malloc(1);
+   cur_dir = malloc(1);
    cur_dir[0] = '\0';
    grow=1;
    
    for (i = 0; i != items; i++) {
       grow+=strlen(dirarray[i])+1;
-      cur_dir = (char *)realloc(cur_dir,grow);
+      cur_dir = realloc(cur_dir,grow);
       strcat(cur_dir,dirarray[i]);
       if ((strcmp(dirstr,"/") != 0 && i == 0) || i > 0)
           strcat(cur_dir,"/");
@@ -386,7 +386,7 @@ frcode(char *dir_path, char *filename, struct stat statres)
    int i;
    FILE *fd;   
    
-   cur_path = (char *)malloc(strlen(dir_path)+strlen(filename)+1);
+   cur_path = malloc(strlen(dir_path)+strlen(filename)+1);
    cur_path[0]='\0';
    strcat(cur_path,dir_path);
    strcat(cur_path,filename);
@@ -405,12 +405,12 @@ frcode(char *dir_path, char *filename, struct stat statres)
       t_num = t_num + fr_num;            
       
       ptr2 = strchr(ptr1,'\0');
-      new_path=(char *)malloc((ptr2-ptr1)+1);
+      new_path=malloc((ptr2-ptr1)+1);
       new_path[0] = '\0';
       strncat(new_path,ptr1,ptr2-ptr1);         
 
    } else {
-      new_path = (char *)malloc(strlen(cur_path)+1);
+      new_path = malloc(strlen(cur_path)+1);
       new_path[0] = '\0';
       strcat(new_path,cur_path);
       t_num = strlen(new_path);
@@ -419,7 +419,7 @@ frcode(char *dir_path, char *filename, struct stat statres)
    if (tmp_path != NULL)
        free(tmp_path);
    
-   tmp_path = (char *)malloc(strlen(cur_path)+1);
+   tmp_path = malloc(strlen(cur_path)+1);
    tmp_path[0] = '\0';
    strcat(tmp_path,cur_path);
    
@@ -485,13 +485,13 @@ create_db(char *dirstr)
    if (dirstr) {
        if (dirstr[strlen(dirstr)-1] == '/') dirstr[strlen(dirstr)-1] = '\0';
    } else {
-      dirstr = (char *)malloc(2);
+      dirstr = malloc(2);
       dirstr[0]='/';
       dirstr[1]='\0';
    }
 
    
-   listptr->name = (char *)statmalloc(strlen(dirstr)+1);
+   listptr->name = statmalloc(strlen(dirstr)+1);
    listptr->name[0] = '\0';
    strcat(listptr->name,dirstr);
    listptr->st_uid = statres.st_uid;
@@ -640,7 +640,7 @@ decode_db(char *str)
 
    slevel = getc(fd);
 	
-	codedpath = (char *)malloc(MIN_BLK);
+	codedpath = malloc(MIN_BLK);
 	codedpath[0] = '\0';
    ptr=codedpath;
    while ((code_num = getc(fd)) != EOF) {
@@ -664,7 +664,7 @@ decode_db(char *str)
          grow++;
 			pathlen++;
 			if (grow == 64) {
-				codedpath = (char *)realloc(codedpath,pathlen+MIN_BLK);
+				codedpath = realloc(codedpath,pathlen+MIN_BLK);
 				grow=0;
 			}
          codedpath[pathlen-1] = ch;
@@ -719,7 +719,10 @@ main(int args, char **argv)
    
    if (args < 2) 
        usage();
-   
+
+	if (!check_dir())
+		 return(1);	
+	
    UID = getuid();        
    GID = getgid();      
    
@@ -745,7 +748,7 @@ main(int args, char **argv)
       case 'U':
          SPECDIR=1;
          optlen=strlen(optarg);
-         spec_dir = (char *)malloc(optlen+1);
+         spec_dir = malloc(optlen+1);
          memmove(spec_dir,optarg,optlen);
          spec_dir[optlen] = '\0';         
          break;
